;
; IDL Event Callback Procedures
; MSP_SAT_eventcb
;
; Generated on:	04/05/2015 12:15.46
; v1.0		DPC		20150405
; v3.0      DPC     20150412 - fixed most bugs
; v3.1      DPC     20150415 - fixed redshift bug that trimmed lines from being displayed
;
;-----------------------------------------------------------------
pro MSP_SAT_Realize, wWidget
		;
	;	get key window information
	;
	WIDGET_CONTROL, wWidget, GET_UVALUE=PTR
	;
;	;	need to pass an event structure:
;	;
	event = {top: wWidget}
	;
	;	main base - change name
	;
	wid_tlb = WIDGET_INFO(wWidget, find_by_uname="MSP_SAT_TLB")
	title = 'Mimir Software Package - Spectrum Annotator/Analyzer Tool (MSP_SAT) '
	title += 'V3.1 20150415'
	WIDGET_CONTROL, wid_tlb, BASE_SET_TITLE=title
	;
	;	window type
	;
	(*PTR).win_type = !D.name
	;
	;	fetch plot window WID
	;
	plot_wid = WIDGET_INFO(wWidget, FIND_BY_UNAME="Plot_Area")
	(*PTR).plot_wid = plot_wid
	;
	;	and turn off the events generated in the plot window
	;
	WIDGET_CONTROL, plot_wid, DRAW_MOTION_EVENTS=0
	WIDGET_CONTROL, plot_wid, DRAW_BUTTON_EVENTS=0
	;
	;	set color table
	;
	device, decomposed=0
	tvlct, [0, 255, 255,   0,   0, 100, 255, 128, 255], $
		   [0, 255,   0, 255,   0, 200,   0, 128, 255], $
		   [0, 255,   0,   0, 255, 200, 255,   0,   0], 10
	color_names = ['black','white','red','green','blue','cyan','magenta','olive','yellow']
	colors = [10,11,12,13,14,15,16,17,18]
	;
	(*PTR).color_names = color_names
	(*PTR).colors = colors
	;
	wid = WIDGET_INFO(wWidget, FIND_BY_UNAME="Baseline_Droplist")
	WIDGET_CONTROL, wid, SET_VALUE='Zero'
	WIDGET_CONTROL, wid, SET_DROPLIST_SELECT=0
	(*PTR).plot_zero_line = 1
	;
	(*PTR).droplist_colors = (*PTR).color_names
	wid = WIDGET_INFO(wWidget, FIND_BY_UNAME="Spectrum_Color_Droplist")
	WIDGET_CONTROL, wid, SET_VALUE=(*PTR).droplist_colors
	WIDGET_CONTROL, wid, SET_DROPLIST_SELECT=1
	(*PTR).p_color = 'white'
	;
	wid = WIDGET_INFO(wWidget, FIND_BY_UNAME="Background_Color_Droplist")
	WIDGET_CONTROL, wid, SET_VALUE=(*PTR).droplist_colors
	WIDGET_CONTROL, wid, SET_DROPLIST_SELECT=0
	(*PTR).b_color = 'black'
	;
	wid = WIDGET_INFO(wWidget, FIND_BY_UNAME="Redshift_Textbox")
	WIDGET_CONTROL, wid, SET_VALUE=STRING( (*PTR).redshift, format='(F8.4)')
	;
	wid = WIDGET_INFO(wWidget, FIND_BY_UNAME='Cursor_Button')
	WIDGET_CONTROL, wid, SET_BUTTON=(*PTR).cursor_flag
	cursor_off, Event
	;
	;	load the line list and default selections for the lines
	;
	load_line_list, Event
	;
	;
	;	erase it and write a message there
	;
	plot_print,event,['Mimir Software Package:',$
		'Spectrum Annotation/Analysis Tool','V3.0  20150415','Initialized']
end
;
;-----------------------------------------------------------------
;
pro MSP_SAT_PostCreation, wWidget, _EXTRA=_VWBExtra_

    ;   set up and clear main structure
    ;
    ;
    main_struct =  {plot_wid:       0L,      		$         ; wid of plotting window
                    ;
                    draw_button_focus:	'none',		$		  ; which routine turned on button events in the draw window
                    										  ;    none = none yet; left, right, start, stop, x, y
                    cursor_focus:	'none',		$		  ; same
                    cursor_x:		0.0,			$		  ; cursor x value buffer
                    cursor_y:		0.0,			$		  ; cursor y value buffer
                    cursor_button:	  0L,			$		  ; 1 = pressed, 0 = released
                    ;
                    cursor_save_x:	0.0,			$		  ; cursor x value, in microns
                    cursor_save_y:	0.0,			$		  ; cursor y value, in flux
                    ;
                    draw_x_size:	0.0,			$		  ; full size of the draw window along x, in pixels
                    draw_y_size:	0.0,			$		  ;
                    plot_x_offset:	0.0,			$		  ; offset of LL x of plot axis, in pixels, from draw window corner
                    plot_y_offset:	0.0,			$		  ;           LL y
;                    plot_x_size:	0.0,			$		  ; length of plotted x axis, in pixels
;                    plot_y_size:	0.0,			$		  ;                   y
  					;
  					plot_panels:	  2,			$		  ; number of plotting sub-panels (1-4)
  					plot_max_panels:  4,			$		  ; max number of plotting sub-panels
  					plot_panel_names: ['IZ','J','H','K'],	$ ;
  					plot_short_wave:  [0.82, 1.16, 1.45, 1.90],$ ; bandpass short wave limits per sub-panel
  					plot_long_wave:   [1.16, 1.35, 1.78, 2.50],$ ;   same, long wave limits
  					;
  					panel_wave_limits: fltarr(4,2), $		  ; start, stop wavelengths to be plotted in each panel
  					;
  					n_xsegments:	  0L,			$		  ; number of plot+buffer segments to be plotted
  					x_segment: fltarr(7,4),			$		  ; for up to 7 segments; 0-start wave,1-end wave; 2-start pix; 3-end pix
  					segment_buffer:   0.04,			$		  ; buffer fraction of a segment, in %
                    ;
                    plot_x_scale:	0.0,			$		  ; x plot scale, in normalized coordinates per channel
                    plot_x_wscale:	0.0,			$		  ; x plot scale, in microns per normalized coordinate
                    plot_y_scale:	0.0,			$		  ; y plot scale, in flux per normalized coordinates
                    position: [0.,0.,1.,1.],		$		  ; save data plot limits in normalized coordinates from plot region
                    ;
                    color_names:	strarr(9),		$		  ; names of plotting colors
                    colors:			intarr(9),		$		  ; corresponding color numbers
                    ;
                    filename:		'',				$		  ; filename of the spectrum read, stripped of directory name
                    dirname:		'',				$		  ; directory name of the spectrum read
                    ;
                    spectrum_ready: 0L,				$		  ; if 1, combined spectrum ready for plotting, else not
					spectrum_nch:	0L,				$		  ; number of channels in the spectrum
                    spectrum_wave:	fltarr(5000),	$		  ; space for the wavelengths of each channel
                    spectrum_flux:	fltarr(5000),	$		  ; space for flux in each channel
                    spectrum_sflux: fltarr(5000),	$		  ; space for flux_uncertainty in each channel
                    spectrum_color:	intarr(5000),	$		  ; color to plot each channel in the spectrum (white = normal; red = overlap)
                    ;
                    header:			PTR_NEW(/alloc),$		  ; FITS header
                    ;
                    ;	what to annotate
                    ;
                    show_atomic_h:		0L,			$		  ; flag
                    show_atomic_he: 	0L,			$
					show_molecular_h:	0L,			$
					show_atomic_mg:		0L,			$
					show_atomic_fe:		0L,			$
					show_atomic_si:		0L,			$
					show_atomic_al:		0L,			$
					show_atomic_na:		0L,			$
					show_atomic_ca:		0L,			$
					show_ionic_fe:		0L,			$
					show_ionic_n:		0L,			$
					show_ionic_he:		0L,		    $
					show_molecular_co:	0L,			$
					show_molecular_13co:0L,			$
					show_molecular_oh:	0L,			$
					;
					cool_stars_flag:	0L,			$
					warm_stars_flag:	0L,			$
					hot_stars_flag:		0L,			$
					;
					emission_flag:		0L,			$
					shocks_flag:		0L,			$
                    ;
                    ;	space for the line list
                    ;
                    n_species:			0L,		    $		  ; number of line list species
                    species:PTR_NEW(/alloc),		$ 	  	  ; structures, with name, line waves PTR, line names PTR
                    ;
                    ;	character size and rotation for annotation
                    ;
                    a_chsz:			   0.85,			$		  ; relative to ych size
                    a_rot:			   90.0,		$		  ; deg, CCW
                    a_thick:		    1.0,		$		  ; thickness
                    ;
                    redshift:			0.0,		$		  ; default redshift
                    ;
                    cursor_flag:		0L,			$		  ; 1 = cursor is on; 0 = cursor is off
                    cursor_wave:		0.0,		$		  ; current cursor wavelength
					;
                    default_left:		0.0,		$		  ; initial left plot limit, in microns
                    default_right:		0.0,		$		  ; initial right plot limit, in microns
                    default_top:		0.0,		$		  ; initial top plot limit. If zero, use max+5%
                    default_bottom:		0.0,		$		  ; initial bottom plot limit. If zero, use min-5%
                    ;
                    left:				0.0,		$		  ; current left plot limit, in microns
                    right:			    0.0,		$		  ; current right plot limit, in microns
                    top:				0.0,		$		  ; current top plot limit. If zero, use max+5%
                    bottom:				0.0,		$		  ; current bottom plot limit. If zero, use min-5%
                    ;
                    left_next:			0.0,		$		  ; requested next left limit wavelength
                    right_next:			0.0,		$		  ; requested next right limit wavelength
                    top_next:			0.0,		$		  ;
                    bottom_next:		0.0,		$
                    ;
                    plot_baseline:		0L,			$		  ; flag for plotting baseline - 0 = none; 1 = one of three
                    plot_zero_line:		0L,			$		  ; if 1 - plot dashed zero line
                    plot_unity_line:	0L,			$		  ; if 1 - plot dashed unity line
                    plot_other_line:	0L,			$		  ; if 1 - plot other baseline line
                    ;
                    plot_uncertainties:	0L,			$		  ; if 1 - plot flux uncertainties also
                    rescale_y_flag:		0L,			$		  ; if 1 - rescale y values on replot
                    this_offset:		0.0,		$		  ; current offset
                    ;
                    ;	droplist colors
                    ;
                    droplist_colors: strarr(9),		$		  ; default colors on color droplists
                    ;
                    ;	save to disk
                    ;
                    win_type:			'',			$		  ; windowing device name
                    write_postscript_flag:	0L,		$		  ; flag to identify the need to write a postscript file
                    filename_postscript: '',		$		  ; filename to write to
                    write_jpeg_flag:	0L,			$		  ; same, for jpeg
                    filename_jpge:		 '',		$		  ; filename for jpeg file
                    ;
                    ;	plot stuff
                    ;
                    p_ch:				1.0,		$		  ; plotting charactersize
                    p_color:		     '',		$		  ; spectrum plotting color
                    b_color:		     ''         $         ; spectrum background color
                    }
    ;
	;
	;------------------------------------------------------------------------
    ;
    PTR = PTR_NEW(main_struct,/NO_COPY)
    ;
    WIDGET_CONTROL, wWidget, SET_UVALUE=PTR
end
;
;-----------------------------------------------------------------
pro Exit_Asserted, Event
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR
	;
	; free the pointer
	;
	PTR_FREE, PTR
	;
	WIDGET_CONTROL, Event.top, /DESTROY
end
;
;-------------------------------------------------------------------------
function get_color, Event, name
	;
	;	returns color index, given name
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR
	;
	ind = WHERE(name eq (*PTR).color_names,n)
	if(n eq 1) then begin
		clr = ind[0]+(*PTR).colors[0]
	endif else begin
		clr = -1
	endelse
	;
;	print, 'in get_color, name = ',name,' returned index = ',clr
	;
	return, clr
end
;
;-----------------------------------------------------------------
;
pro plot_print,event,str
	;
	;	prints the str into the plotting window
	;	if str is an array, stacks the strings on the plot window
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR
	;
	nrows = N_ELEMENTS(str)
	;
	if(nrows eq 1) then begin
		this_str = [str]
		y = [0.5]
	endif else begin
		this_str = str
		y = findgen(nrows)*(-0.5/float(nrows-1))+0.75
	endelse
	;
	ERASE
	plot, [0,1],[0,1],/nodata,xs=4,ys=4
	;
	for i = 0, nrows-1 do begin
		;
		XYoutS, 0.5,y[i],str[i],align=0.5,charsize=1.5
	endfor
	;
end
;
;-----------------------------------------------------------------
; Empty stub procedure used for autoloading.
;
pro MSP_SAT_eventcb
end
;
;-----------------------------------------------------------------
pro Get_Spectrum, Event
	;
	;	fetch pointer to data structure
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR
	;
	;	if the current directory is not set, then just throw up a file chooser,
	;	but if set, point the file chooser to that directory
	;
	restart:
	;
;	print,'Dirname = ',(*PTR).dirname
	;
	filters = ['*.fits','*.FITS','*.fts','*.fit']
	if( (*PTR).dirname eq '') then begin
		filename = Dialog_Pickfile(title='Select SpeX-type Spectrum', $
					FILTER=filters)
		this_dir = FILE_DIRNAME(filename)
		(*PTR).dirname = this_dir
	endif else begin
		filename = Dialog_Pickfile(title='Select SpeX-type Spectrum',$
			PATH = (*PTR).dirname, FILTER=filters)
	endelse
	;
	; read file
	;
	image = readfits(filename,header,/silent)
	;
	;	save header
	;
	(*(*PTR).header) = header
	;
	;	if, ok, load some stuff
	;
	s = size(image)
	nx = s[1]					; channel dimension
	ny = s[2]					; should be 3
	;
	if(ny ne 3 or nx lt 10) then begin
		plot_print,Event,['Bad SpeX FITS File','Skipping this file']
		goto, not_read
	endif
	;
	;
	(*PTR).spectrum_nch = nx				; channels in this spectrum
	;
	;	then, store the vectors
	;
	(*PTR).spectrum_wave[0:nx-1] = REFORM(image[*,0])
	(*PTR).spectrum_flux[0:nx-1] = REFORM(image[*,1])
	(*PTR).spectrum_sflux[0:nx-1] = REFORM(image[*,2])
		;
		;   clean NaNs - ** hold off on clearing them for now
		;
;		ind_f = WHERE(~FINITE((*(*PTR).spectrum_flux)[*]),count)
;		if(count ne 0) then begin
;	;	    print,"Found NaNs, count = ",count
;		    ind_good = WHERE(FINITE((*(*PTR).spectrum_flux)[*]))
;		    zz = median_filtered_mean((*(*PTR).spectrum_flux)[ind_good])
;		    (*(*PTR).spectrum_flux)[ind_f] = zz[0]
;		    (*(*PTR).spectrum_sflux)[ind_f] = zz[0]
;	;	    print, "reset to mean of ",zz[0]
;		endif
		;
		;	clear some buffers back to default values
		;
	(*PTR).left = (*PTR).default_left
	(*PTR).left_next = (*PTR).default_left
	;
	(*PTR).right = (*PTR).default_right
	(*PTR).right_next = (*PTR).default_right
	;
	(*PTR).top = (*PTR).default_top
	(*PTR).top_next = (*PTR).default_top
	;
	(*PTR).bottom = (*PTR).default_bottom
	(*PTR).bottom_next = (*PTR).default_bottom
		;
	not_read:
	;
	plot_spectrum,event
end
;
;-----------------------------------------------------------------
pro plot_spectrum, Event
		;
	;	plots the specta, by color
	;
	;	includes adding offsets
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR
	;
	;	what type of plot? are either the postscript or jpeg flags set?
	;
	iplot = 0		; default, win or X
	if((*PTR).write_postscript_flag eq 1) then iplot = 1
	if((*PTR).write_jpeg_flag eq 1) then iplot = 2
	;
	;	if iplot = 0, get current windowing system info
	;
	case iplot of
		0: begin
				;
				set_plot, (*PTR).win_type
				;
				;	only one window, so shouldn't have to call WINDOW
				;
				;    device, decomposed=0	- did this in REALIZE - do again?
				;    arrowsize = 15
			    !p.thick=1
			    !x.thick=1
			    !y.thick=1
			    !p.charthick=1
			    !p.charsize=2
			    ;
			    (*ptr).a_thick = 1
			    if((*PTR).b_color eq 'black') then a_color = 'white' else a_color = 'black'
				;
			    (*PTR).color_names = ['black','white','red','green','blue','cyan','magenta','olive','yellow']
		   end
		1: begin
				set_plot,'PS'
			    ;device,/landscape
			    device,/color
			    device,bits=8
			    device,encap=1
			    device,file=(*PTR).filename_postscript
			    device,/inches,XSIZE=7.5,YSIZE=6.0
;			    black = 10
;			    arrowsize=300
			    !p.thick=2
			    !x.thick=4
			    !y.thick=4
			    !p.charthick=2
			    !p.charsize=2
			    (*PTR).a_thick=3
;			    ch_label = 1.5
;			    offset = 0.012
;			    scale = 0.93*0.65
;			    xj0 = 0.17
				if((*PTR).b_color eq 'white') then a_color = 'black' else a_color = 'white'
				;
				(*PTR).color_names = ['white','black','red','green','blue','cyan','magenta','olive','yellow']
		   end
		2: begin
		   end
	endcase
	;
	;   initialize - left, right, top, bottom - based on spectral wavelengths and fluxes
	;
	this_nch = (*PTR).spectrum_nch
	;
	;	if no channels - no spectrum yet loaded, skip out
	;
	if(this_nch eq 0) then goto, skip_out
	;
	all_wave = (*PTR).spectrum_wave[0:this_nch-1]
	all_flux = (*PTR).spectrum_flux[0:this_nch-1]
	all_sflux = (*PTR).spectrum_sflux[0:this_nch-1]
	;
	max_nch = N_ELEMENTS(all_wave)
	spec_nch = this_nch
	;
	;	if x next limits set to zero, find limits, else use saved values
	;
	if((*PTR).left eq 0.0) then begin
		xmin = min(all_wave)
		(*PTR).left = xmin
		ich0 = 0
	endif else begin
		xmin = (*PTR).left
		ind = WHERE((*PTR).spectrum_wave[0:spec_nch-1] lt xmin,nind)
		if(nind gt 0) then begin
			ich0 = ind[nind-1]				; for the combined spectrum
		endif else begin
			ich0 = 0
		endelse
	endelse
	;
	if((*PTR).right eq 0.0) then begin
		xmax = max(all_wave)
		(*PTR).right = xmax
		ich1 = N_ELEMENTS(all_wave)-1
	endif else begin
		xmax = (*PTR).right
		ind = WHERE((*PTR).spectrum_wave[0:spec_nch-1] gt xmax,nind)
		if(nind gt 0) then begin
			ich1 = ind[0]
		endif else begin
			ich1 = spec_nch-1
		endelse
	endelse
	;
	replot = 1
	if((*PTR).bottom eq 0.0) then begin
		if((*PTR).rescale_y_flag eq 1L) then ymin = MIN((*PTR).spectrum_flux[ich0:ich1]) else ymin = MIN(all_flux)
		(*PTR).bottom = ymin
		replot = 0
	endif else begin
		ymin = (*PTR).bottom
	endelse
	;
	if((*PTR).top eq 0.0) then begin
		if((*PTR).rescale_y_flag eq 1L) then ymax = MAX((*PTR).spectrum_flux[ich0:ich1]) else ymax = MAX(all_flux)
		(*PTR).top = ymax
		replot = 0
	endif else begin
		ymax = (*PTR).top
	endelse
	;
	;	if original spectrum, provide a bit of room at top and bottom,
	;	but if a replot, skip this
	;
	dy = (ymax - ymin)
	;
	if(replot eq 0) then begin
		ymax = ymax + 0.05 * dy
		ymin = ymin - 0.05 * dy
	endif
	;
	(*PTR).top = ymax
	(*PTR).bottom = ymin
	;
	;		plot limits found. Plot boundary, then loop through spectra
	;
	xxplot = [xmin, xmax]
	yyplot = [ymin, ymax]
	;
	;   load plot vectors - keeping only those in the selected wavelength range
	;
	xplot = (*PTR).spectrum_wave[ich0:ich1]
	yplot = (*PTR).spectrum_flux[ich0:ich1]
	splot = (*PTR).spectrum_sflux[ich0:ich1]
	;
	;	trap, save plot limits
	;
	;
	;	plot data limits, normalized, and within plot region boundary
	;
	pxrange = [0.10,0.98]
	pyrange = [0.08,0.98]
	;
	(*PTR).position=[pxrange[0],pyrange[0],pxrange[1],pyrange[1]]

	!P.position=[pxrange[0],pyrange[0],pxrange[1],pyrange[1]]	; plot data region
	!P.region=[0.,0.,1.,1.]										; plot region
	;
	clip = !P.clip
	(*PTR).plot_x_offset = pxrange[0]						; starting x location, normalized
	(*PTR).plot_y_offset = pyrange[0]						; starting y "
;	(*PTR).plot_x_size = pxrange[1] - pxrange[0] 		; x size of plot region
;	(*PTR).plot_y_size = pyrange[1] - pyrange[0] 		; y size of plot region
	;
;	print, 'Plot window X limits [normalized] = ',pxrange[1],' ',pxrange[0]
	;
	;	plot scale factors
	;
;	(*PTR).plot_x_scale = ( (*PTR).right - (*PTR).left ) / (*PTR).plot_x_size		; revisit when segments established
	(*PTR).plot_y_scale = ( (*PTR).top - (*PTR).bottom ) / (pyrange[1] - pyrange[0]) ; y per normalized coords
	;
;	stop
	;==========================================================
	;
	;		Up to four sub-panels of plots - for IZ, J, H, Ks bands
	;
	;	Determine how many subplots, based on wavelength range
	;	Axes appear in first and final subplots
	;
	;
	npanels = 0
	;
	;	check each subpanel's wave limits against the plot limits to
	;	see how many subpanels to plot, and over what wave limits.
	;	 Pay attention to zooming.
	;
	panels = intarr(4)
	;
	first_panel = -1
	last_panel = 5
	;
	;panel_wave_limits = fltarr(4,2)			; start, stop
	;
	total_wave = 0.0
	;
	for ip = 0, 3 do begin
		;
		short_wave = (*PTR).plot_short_wave[ip]			; this band's short,long wave limits
		long_wave = (*PTR).plot_long_wave[ip]
		;
		ind = WHERE(xplot ge (short_wave > xmin) and xplot le (long_wave < xmax),nind)
		;
		if(nind gt 0) then begin
			panels[ip] = 1				; some data will be plotted in this panel
			if(first_panel eq -1) then first_panel = ip
			last_panel = ip
			(*PTR).panel_wave_limits[ip,*] = [xplot[ind[0]],xplot[ind[nind-1]]]		; wavelengths to plot in this panel
			total_wave += (*PTR).panel_wave_limits[ip,1] - (*PTR).panel_wave_limits[ip,0] ; add to wavelength count
		endif
	endfor
	;
	npanels = FIX(TOTAL(panels))
	;
;	print, 'number of subpanels to plot = ',npanels
;	print, 'first, last panels = ',first_panel,last_panel
;	print, 'total bandwidth to plot (in microns) = ',total_wave
;	print, 'short - long wavelength limits (in microns) = ',xmax-xmin
	;
	dwave = xplot[1] - xplot[0]
	n_total_wave = CEIL(total_wave / dwave)			; number of spectral channels
	;
	n_buffer = (npanels - 1)
	if(n_buffer gt 0) then begin
		;
		;	number of buffer channels per buffer region is a fixed percentage of the
		;	number of spectral channels
		;
		buffer_channel_width = ROUND(n_total_wave * (*PTR).segment_buffer / float(n_buffer))
	endif else begin
		buffer_channel_width = 0
	endelse
	;
;	print, 'n_buffer regions = ',n_buffer
;	print, 'buffer channels per buffer region = ',buffer_channel_width
;	print, 'bandwidth per buffer region = ',buffer_channel_width * dwave
	;
	(*PTR).n_xsegments = npanels + (npanels-1)		; number of segments to plot = bands + buffers
	;
	total_channels = n_total_wave + buffer_channel_width * n_buffer
	;
;	print, 'Spectral Channels = ',n_total_wave,' Buffer Channels = ',buffer_channel_width * n_buffer,$
;			'  Total Channels = ',total_channels
	;
	;	compute start/stop wavelength and plot locations for each segment
	;
	(*PTR).plot_x_scale = (pxrange[1] - pxrange[0]) / float(total_channels); normalized distance per channel
	(*PTR).plot_x_wscale =  dwave / (*PTR).plot_x_scale					   ; microns per normalized distance
	;
	this_panel = first_panel
	;
	for is = 0, (*PTR).n_xsegments-1 do begin
		;
		;	every even segment is spectrum, odd is buffer
		;
		if(2*floor(is/2) eq is) then begin
			;
			;	spectra first
			;
			if(is eq 0) then begin
				seg_type = 'spec'
				(*PTR).x_segment[is,0] = (*PTR).panel_wave_limits[this_panel,0]										; short wave
				(*PTR).x_segment[is,1] = (*PTR).panel_wave_limits[this_panel,1]		; long wave
				;
				;	number of channels
				;
				segment_channels = ROUND(((*PTR).x_segment[is,1] - (*PTR).x_segment[is,0])/dwave)
				;
				(*PTR).x_segment[is,2] = pxrange[0]									; short pix
				(*PTR).x_segment[is,3] = pxrange[0] + (*PTR).plot_x_scale*float(segment_channels-1)
				;
				this_panel++
			endif else begin
				seg_type = 'spec 2'
				(*PTR).x_segment[is,0] = (*PTR).panel_wave_limits[this_panel,0]
				(*PTR).x_segment[is,1] = (*PTR).panel_wave_limits[this_panel,1]
				;
				segment_channels = ROUND(((*PTR).x_segment[is,1] - (*PTR).x_segment[is,0])/dwave)
				;
				(*PTR).x_segment[is,2] = (*PTR).x_segment[is-1,3]+(*PTR).plot_x_scale  ; bump by one channel
				(*PTR).x_segment[is,3] = (*PTR).x_segment[is,2]+(*PTR).plot_x_scale*float(segment_channels-1)
				;
				this_panel++
			endelse
		endif else begin
			;
			;	then buffers
			;
			seg_type = 'buf'
			(*PTR).x_segment[is,0] = (*PTR).panel_wave_limits[this_panel-1,1]
			(*PTR).x_segment[is,1] = (*PTR).x_segment[is,0] + dwave*(buffer_channel_width-1)
			(*PTR).x_segment[is,2] = (*PTR).x_segment[is-1,3]+(*PTR).plot_x_scale ; bump by one channel
			(*PTR).x_segment[is,3] = (*PTR).x_segment[is,2]+(buffer_channel_width-1)*(*PTR).plot_x_scale  ; bump by one channel
		endelse
		;
;		print, 'For Segment Type = ',seg_type
;		print, '   Start/Stop Wavelength = ',(*PTR).x_segment[is,0],' ',(*PTR).x_segment[is,1]
;		print, '   Start/Stop Plot Normalized coords = ',(*PTR).x_segment[is,2],' ',(*PTR).x_segment[is,3]
	endfor
;
	!P.MULTI=[0,npanels]
	;
	!x.title = ''
	;!y.title = '!17Flux!17'
	;
	ych = 0.5
	xch = 0.5
	;
	;	for every possible panel
	;
	this_panel = 0
	;
	;	data range - data units
	;
	yrange = yyplot
	yr = yrange
	;
	dxr = pxrange[1]-pxrange[0]
;	dxp = clip[2] - clip[0]
	;
;	print, 'left, right normalized limits and difference = ',pxrange[0],' ',pxrange[1],' ',dxr
	;
	start_x = pxrange[0]
	;
	xleft = 100.0
	xright = -100.0
	;
	for ip = first_panel, last_panel do begin
		;
		this_seg = 2*(ip-first_panel)
		pix_0 = (*PTR).x_segment[this_seg,2]		; fractional, not pixels now
		pix_1 = (*PTR).x_segment[this_seg,3]
		d_pix = pix_1 - pix_0
		;
		xrange = [(*PTR).panel_wave_limits[ip,0],(*PTR).panel_wave_limits[ip,1]]
		;
;		print,' Panel ',ip,' Wave range = ',xrange,' location range = ',pix_0,' ',pix_1
			;
		;	how much to plot?
		;
		panel_width = d_pix
		;
		;	if there is to be a buffer, compute its width
		;
		if( (*PTR).n_xsegments gt 2*(ip-first_panel)) then begin
			b_pix_0 = (*PTR).x_segment[this_seg+1,2]
			b_pix_1 = (*PTR).x_segment[this_seg+1,3]
			b_d_pix = b_pix_1 - b_pix_0
			;
			buffer_width = b_d_pix
		endif else begin
			buffer_width = 0.0
		endelse
		;
		if(ip eq first_panel) then begin

		 	if(ip eq last_panel) then begin
				ys=1		; both axes
				x0 = start_x
				x1 = pxrange[1]
	 		endif else begin
				ys=9		; left axis
				;
				x0 = start_x
				x1 = x0+panel_width
				;
				start_x += panel_width+buffer_width
			endelse
		endif else begin
			ys=5							; no axes
			;
			x0 = start_x
			x1 = x0+panel_width
			;
			start_x += panel_width+buffer_width
		endelse
		;
		y0 = pyrange[0]
		y1 = pyrange[1]
		;
		position=[x0,y0,x1,y1]
		;
		ich0 = WHERE(xplot ge xrange[0],n_ich0)
		ich1 = WHERE(xplot le xrange[1],n_ich1)
		;
		if(n_ich0 eq 0 or n_ich1 eq 0) then goto, no_panel
		;
		ich0 = ich0[0]
		ich1 = ich1[n_ich1-1]
		;
		;	x bandwidth
		;
		dxbw = xplot[ich1] - xplot[ich0]
		;
		;	set tick marks - if bandwidth > 0.3 microns, then 0.1
		;								  < 0.3        , then 0.05
		;								  < 0.15       , then 0.025
		;								  < 0.08       , then 0.02
		;								  < 0.04       ; then 0.01
		;
		xti = 0.01 * floor(10.00 * alog10(dxbw) + 14.0)
	    ;
		;
	    plot, xplot[ich0:ich1],yplot[ich0:ich1], xs=1,ys=ys, $
	    	   xr=xrange,yr=yr,psym=10, xmargin=xmargin, $
	           YTICKINTERVAL=yti, POSITION=position, ycharsize=ych, $
	           XTICKINTERVAL=xti,/NODATA, color=get_color(Event,a_color), $
	           background=get_color(Event,(*PTR).b_color),$
	           LINESTYLE=0,xcharsize=xch,thick=!P.thick,xthick=!x.thick,$
	           ythick=!y.thick
	    ;
	    oplot, xplot[ich0:ich1],yplot[ich0:ich1],thick=2*!P.thick, psym=-3,symsize=0.003, $
	    		color=get_color(Event,(*PTR).p_color)
		;
		if(ip eq last_panel) then begin
			;
			;	plot right hand vertical axis on last subpanel
			;
			AXIS, YAXIS=1, yr=yr, YTICKINTERVAL=yti, ys=1, ycharsize=ych, $
					ytickname=[' ',' ',' ',' ',' ',' ',' '],color=get_color(Event,a_color),$
					ythick=!y.thick
		endif
		;
		no_panel:
	endfor
	;
	;	reset to 1 plot per page to add xlabel
	;
	!P.MULTI=0
	xxplot = pxrange		; for normalized plotting
	yyplot = pyrange
	;
	plot, xxplot, yyplot, /noerase, /nodata, xtitle='Observed Wavelength [microns]',$
		position=[pxrange[0],pyrange[0],pxrange[1],pyrange[1]],$
		xs=1,ys=5,xcharsize=xch,xminor=1,xthick=0.0,$
		xticklayout=1,xticks=1,xtickname=[' ',' '],$
		xr=xxplot,yr=yyplot,ycharsize=ych,$
		xmargin=xmargin,color=get_color(Event,a_color),$
		background=get_color(Event,(*PTR).b_color),$
		charthick=!P.charthick
;
;	plot, xxplot, yyplot, XRANGE=[xmin, xmax], YRANGE=[ymin, ymax], XSTYLE=1, $
;		color=11, background=10, thick=1, YSTYLE=1, LINESTYLE=0, PSYM=10, $
;		xtitle='Wavelength [microns]',/nodata
;	;
;	oplot, xplot, yplot,thick=2, psym=-3,symsize=0.003, color=get_color(Event,'white')
	;
	;	if the flag for the flux uncertainty spectrum is also set, draw a light version of it
	;
;	if((*PTR).plot_uncertainties eq 1L) then begin
;		oplot, xplot, splot, thick=1, color=get_color(Event, 'yellow')
;	endif
	;
	if( (*PTR).plot_baseline eq 1) then begin
		;
		;   if unity is between ymax and ymin, plot continuum line
		;
		if(ymax gt 1. and ymin lt 1. and (*PTR).plot_unity_line eq 1L) then begin
		    xz = xxplot
		    yz = [1., 1.]
		    oplot, xz, yz, color=get_color(event,'green'),linestyle=2,thick=2*!P.thick
		endif
		;
		;	dashed zero line, if flag set and zero contained in plot
		;
		if(ymax gt 0. and ymin lt 0. and (*PTR).plot_zero_line eq 1L) then begin
		    xz = xxplot
		    yz = [0., 0.]
		    oplot, xz, yz, color=get_color(event,'blue'),linestyle=2,thick=2*!P.thick
		endif
		;
		;	stub for other baseline display
		;
	endif
	;
;	plot, pxrange,pyrange,/nodata,/noerase,xs=5,ys=5
	;
;	revisit plot limits
;
;	clip = !P.clip
;	(*PTR).plot_x_offset = clip[0]					; starting x pixel
;	(*PTR).plot_y_offset = clip[1]					; starting y pixel
;	(*PTR).plot_x_size = clip[2] - clip[0] + 1		; x size of plot region
;	(*PTR).plot_y_size = clip[3] - clip[1] + 1		; y size of plot region
	;
	;	do the lines annotation
	;
	annotate_lines, Event
	;
	;	plotting done. If not to screen, close device to dispose of plot
	;
	if(iplot gt 0) then begin
		device,/close
	endif
	skip_out:
end
;
;-----------------------------------------------------------------
pro Baseline_Select, Event
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR
	;
	base = Event.index
	;
	case base of
		0:	begin
				(*PTR).plot_zero_line = 1
				(*PTR).plot_unity_line = 0
				(*PTR).plot_other_line = 0
			end
		1:  begin
				(*PTR).plot_zero_line = 0
				(*PTR).plot_unity_line = 1
				(*PTR).plot_other_line = 0
			end
		2:  begin
				(*PTR).plot_zero_line = 0
				(*PTR).plot_unity_line = 0
				(*PTR).plot_other_line = 0
				;
				;	switch on textbox, collect new value
				;
			end
	endcase
end
;-----------------------------------------------------------------
; Text Insert Character Callback Procedure.
; Argument:
;   Event structure:
;
;   {WIDGET_TEXT_CH, ID:0L, TOP:0L, HANDLER:0L, TYPE:0, OFFSET:0L,
;       CH:0B }

;   ID is the widget ID of the component generating the event. TOP is
;       the widget ID of the top level widget containing ID. HANDLER
;       contains the widget ID of the widget associated with the
;       handler routine.

;   OFFSET is the (zero-based) insertion position that will result
;       after the character is inserted. CH is the ASCII value of the
;       character.

;   Retrieve the IDs of other widgets in the widget hierarchy using
;       id=widget_info(Event.top, FIND_BY_UNAME=name)

;-----------------------------------------------------------------
pro Other_Baseline_Textbox, Event

end
;
;----------------------------------------------------------------
pro cursor_on, Event
	;
	;	turn on motion sensitivity in plot window, listen for mouse
	;	click
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR
	;
	plot_wid = (*PTR).plot_wid
	;
	WIDGET_CONTROL, plot_wid, DRAW_BUTTON_EVENTS=1
	WIDGET_CONTROL, plot_wid, DRAW_MOTION_EVENTS=1
	;
	(*PTR).cursor_button = 0
	(*PTR).cursor_flag = 1
	;
end
;-----------------------------------------------------------------
pro Draw_Button, Event
	;
	;	if the mouse button was pressed or released while draw_button_events
	;	is enabled, then disable these events and the motion events
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR
	;
	;	if button still pressed, skip out of routine
	;
	if((*PTR).cursor_button eq Event.type) then begin
;		print, 'button unchanged'
		goto, skip_button
	endif else begin
		;
		;	but if button changed, toggle status
		;
		(*PTR).cursor_button = 1 - (*PTR).cursor_button
	endelse
	;
	;	now, if flag says pressed, process it, else skip out
	;
	if( (*PTR).cursor_button eq 0) then goto, skip_button
	;
	plot_wid = (*PTR).plot_wid
	;
;	WIDGET_CONTROL, plot_wid, DRAW_MOTION_EVENTS=0
;	WIDGET_CONTROL, plot_wid, DRAW_BUTTON_EVENTS=0
	;
	; decide on proper action
	;
;	stop
	cursor_focus = (*PTR).cursor_focus
	;
;	print, 'Entered Draw_Button with button focus = ',button_focus
	;
	case cursor_focus of
		'left':  begin
					;
					;	save the value in the cursor_save_x in the left next buffer
					;
					(*PTR).left_next = (*PTR).cursor_save_x
					;
					;	and draw a dashed cyan line there, using normalized coordinates
					;
					xx = x_line(Event,(*PTR).left_next)		; translate wavelengths to normalized coords
					yy0 = y_line(Event,(*PTR).bottom)
					yy1 = y_line(Event,(*PTR).top)
					;
;					print, 'left - xx = ',xx[0],' ',xx[1]
					;
				;	xx = [1.0,1.0]*(*PTR).left_next
					yy = [yy0[0],yy1[1]]
					dy = yy[1] - yy[0]
					yy[1] -= 0.1*dy
					oplot, xx, yy, psym=-3,symsize=0.003,color=get_color(Event,'cyan'),thick=3,linestyle=2
					xyouts, xx[0],yy[1]+0.05*dy,'L',align=0.5,color=get_color(Event,'cyan'),charsize=1.5,charthick=2
					;
;					print, 'left - plot clipping = ',!P.clip[2],' ',!P.clip[0]
;					print, 'left - plot range = ',!X.RANGE[0],' ',!X.RANGE[1]
					;
					;	then invoke zoom_x with mode =1
					;
					wait, 0.1
					Zoom_X, Event, MODE=1
			     end
		'right': begin
					(*PTR).right_next = (*PTR).cursor_save_x
					yy0 = y_line(Event,(*PTR).bottom)
					yy1 = y_line(Event,(*PTR).top)
					;
					;	and draw a dashed olive line there
					;
					xx = x_line(Event,(*PTR).right_next)
					;
					yy = [yy0[0],yy1[1]]
					dy = yy[1] - yy[0]
					yy[1] -= 0.1*dy
					oplot, xx, yy, psym=-3,symsize=0.003,color=get_color(Event,'olive'),thick=3,linestyle=2
					xyouts, xx[0],yy[1]+0.05*dy,'R',align=0.5,color=get_color(Event,'olive'),charsize=1.5,charthick=2
				;					;
					;
;					print, 'right - plot clipping = ',!P.clip[2],' ',!P.clip[0]
					;
					wait, 0.5
					Zoom_X, Event, MODE=2
				 end
		'bottom': begin
					(*PTR).bottom_next = (*PTR).cursor_save_y
					;
					; load into next region buffer
					;
					yy = y_line(Event,(*PTR).bottom_next)
					;
;					print,' bottom - left, right = ',(*PTR).left,' ',(*PTR).right
					;
					xx0 = x_line(Event,(*PTR).left)
					xx1 = x_line(Event,(*PTR).right)
					;
					xx = [xx0[0],xx1[1]]
					;
					dx = xx[1] - xx[0]
					xx[1] -= 0.1*dx
					xx[0] += 0.1*dx
					;
;					print, 'bottom - xx0 = ',xx0[0],' xx1 = ',xx1[1]
;					print, 'bottom - dx = ',dx
					oplot, xx, yy, psym=-3,symsize=0.003,color=get_color(Event,'cyan'),thick=3,linestyle=2
					xyouts, xx[1]+0.05*dx,yy[0],'B',align=0.5,color=get_color(Event,'cyan'),charsize=1.5,charthick=2
					;
					wait, 0.1
					Zoom_Y, Event, MODE=1
				 end
		'top':  begin
					(*PTR).top_next = (*PTR).cursor_save_y
					;
					yy = y_line(Event,(*PTR).top_next)
					xx0 = x_line(Event,(*PTR).left)
					xx1 = x_line(Event,(*PTR).right)
					;
					xx = [xx0[0],xx1[1]]
					;
					dx = xx[1] - xx[0]
					xx[1] -= 0.1*dx
					xx[0] += 0.1*dx
					oplot, xx, yy, psym=-3,symsize=0.003,color=get_color(Event,'olive'),thick=3,linestyle=2
					xyouts, xx[1]+0.05*dx,yy[0],'T',align=0.5,color=get_color(Event,'olive'),charsize=1.5,charthick=2
					;
					wait, 0.1
					Zoom_Y, Event, MODE=2
				 end
		'none':	 begin
;					print,'Button focus = none. Should not happen'
					cursor_off, Event
				 end
	endcase
	;
	skip_button:
end
;
;---------------------------------------------------------------
function x_line,Event,x_wave
	;
	;	takes a designated wavelength, finds which segment it resides in,
	;	determines its pixel location and returns a normalized [1,1]
	;	vector in normalized units (for plotting)
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR
	;
	x = x_wave
	;
	;	which spectral segment is it in?
	;
	n_seg = (*PTR).n_xsegments
	;
	z = 0.0
	for is = 0, n_seg-1, 2 do begin	; skip over buffer regions
		;
		x0 = (*PTR).x_segment[is,0]		; wavelength start, stop for this plot segment
		x1 = (*PTR).x_segment[is,1]
		;
		if((x+0.00075) ge x0 and (x-0.00075) le x1) then begin
			;
			p0 = (*PTR).x_segment[is,2]	; normalized start for this plot segment
			;
;			print, 'segment is'
;			print, '  x, x0, x1 = ',x, x0, x1
			;
			dx = x - x0
			;
			z = p0 + float(dx) / (*PTR).plot_x_wscale
			;
			goto, no_more
		endif
	endfor
	no_more:
	zz = [1.0,1.0]*z
	return, zz
end
;
;-----------------------------------------------------------------
function y_line,Event,y_height
	;
	;	takes a designated height and returns a normalized [1,1]
	;	vector in normalized units (for plotting)
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR
	;
	y = y_height
	;
	ybot = (*PTR).bottom
	ytop = (*PTR).top
	;
	yr = [(*PTR).position[1],(*PTR).position[3]]
	;
	this_p = ((y - ybot) / (ytop - ybot))*(yr[1]-yr[0])
	this_p += (*PTR).position[1]
	;
	zz = [1.0,1.0]*this_p
	return, zz
end
;
;-----------------------------------------------------------------
pro Draw_Motion, Event
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR
	;
	;	save the X, Y position to the buffer, if focus identified,
	;	echo to proper text window
	;
	(*PTR).cursor_x = Event.x		; in pixels
	(*PTR).cursor_y = Event.y		; in pixels
	;
	;	fetch draw window geometry to be able to compare
	;
	win_geo = WIDGET_INFO(Event.id,/GEOMETRY)
	;
;	stop
	x_win_range = [0,win_geo.scr_xsize-1] ; in pixels
	y_win_range = [0,win_geo.scr_ysize-1]	; in pixels
	;
	dx_win = x_win_range[1] - x_win_range[0]	; in pixels
	dy_win = y_win_range[1] - y_win_range[0]	; in pixels
	;
	;	cursor location in normalized coordinates
	;
	x_normal = float(Event.x - x_win_range[0])/float(dx_win)
	y_normal = float(Event.y - y_win_range[0])/float(dy_win)
	;
	;	must do lookup of segments to find which segment and
	;	which wavelength
	;
	n_seg = (*PTR).n_xsegments
	;
	;	for each segment, see if this x pixel value is contained
	;
	lambda = 0.0
	;
	for is = 0, n_seg-1 do begin
		;
		p0 = (*PTR).x_segment[is,2]
		p1 = (*PTR).x_segment[is,3]
		;
		if(x_normal ge p0 and x_normal le p1) then begin
			;
			;	found it. compute scale, offsets
			;
			lambda = (x_normal - p0) * (*PTR).plot_x_wscale
			lambda += (*PTR).x_segment[is,0]
			goto, out_loop
		endif
	endfor
	out_loop:
	;
	;	save the wavelength in the structure
	;
	(*PTR).cursor_save_x = lambda
	;
	;	and display to the cursor text window, if the cursor flag is set
	;
	if((*PTR).cursor_flag eq 1) then begin
		wid = WIDGET_INFO(Event.top, FIND_BY_UNAME="Wavelength_Textbox")
		WIDGET_CONTROL, wid, SET_VALUE=STRING(lambda,format='(2x,F8.4)')
	endif
	;
	height = (y_normal - (*PTR).plot_y_offset) * (*PTR).plot_y_scale
	height += (*PTR).bottom
	;
	;	save the height
	;
	(*PTR).cursor_save_y = height
	;
;	print, 'x,y = ',Event.x,Event.y,'   X,Y normal = ',x_normal,' ',y_normal,$
;		' segment = ',is,' wavelength, height = ',lambda,height
;	stop
	;
end
;
;-----------------------------------------------------------------
pro cursor_off, Event
	;
	;	turn of motion sensitivity in plot window
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR
	;
	plot_wid = (*PTR).plot_wid
	;
	WIDGET_CONTROL, plot_wid, DRAW_BUTTON_EVENTS=1
	WIDGET_CONTROL, plot_wid, DRAW_MOTION_EVENTS=0
	;
	(*PTR).cursor_button = 0
	(*PTR).cursor_flag = 0
	;
end
;
;-----------------------------------------------------------------
pro Zoom_X, Event, MODE=mode
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR
	;
	;	if mode arg is missing, this is the first call, so turn on
	;	cursor and get ready for setting the left boundary
	;
	if(N_ELEMENTS(mode) eq 0) then mode = 0
	;
	case mode of
		0:	begin
				(*PTR).cursor_focus = 'left'
				cursor_on, Event
			end

		1:  begin		; came in with left focus - put in buffer, then
						; switch to right focus
				;
				(*PTR).cursor_focus = 'right'
				wait, 0.1
				cursor_on, Event
			end

		2:	begin		; came in with right focus - put in buffer, turn
						; off cursor, replot
						;
				;
				(*PTR).cursor_focus ='none'
				wait, 0.5
				cursor_off, Event
				;
				(*PTR).left = (*PTR).left_next
				(*PTR).right = (*PTR).right_next
				;
				plot_spectrum, Event
			end
	endcase
	;
end
;
;-----------------------------------------------------------------
pro Unzoom_X, Event
	;
	;	reset to default (find limits from raw spectrum)
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR
	;
	(*PTR).left = 0.0
	(*PTR).right = 0.0
	;
;	print, 'see unzoom button press'
	;
	plot_spectrum, Event
	;
end
;
;-----------------------------------------------------------------
pro Zoom_Y, Event, MODE=mode
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR
	;
	;	if mode arg is missing, this is the first call, so turn on
	;	cursor and get ready for setting the left boundary
	;
	if(N_ELEMENTS(mode) eq 0) then mode = 0
	;
	case mode of
		0:	begin
				(*PTR).cursor_focus = 'bottom'
				cursor_on, Event
			end

		1:  begin		; came in with left focus - put in buffer, then
						; switch to right focus
				;
				(*PTR).cursor_focus = 'top'
				wait, 0.1
				cursor_on, Event
			end

		2:	begin		; came in with right focus - put in buffer, turn
						; off cursor, replot
						;
				;
				(*PTR).cursor_focus ='none'
				wait, 0.5
				cursor_off, Event
				;
				(*PTR).bottom = (*PTR).bottom_next
				(*PTR).top = (*PTR).top_next
				;
				plot_spectrum, Event
			end
	endcase
	;
end
;
;-----------------------------------------------------------------
pro Unzoom_Y, Event
	;
	;	reset to default (find limits from raw spectrum)
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR
	;
	(*PTR).bottom = 0.0
	(*PTR).top = 0.0
	;
;	print, 'see unzoom button press'
	;
	plot_spectrum, Event
	;
end
;
;-----------------------------------------------------------------
pro Colorize_Spectrum, Event
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE=ptr
	;
	;	get color indicated by Event.index
	;
	(*PTR).p_color = (*PTR).droplist_colors[Event.index]
	;
	plot_spectrum, Event
end
;
;-----------------------------------------------------------------
pro Colorize_Background, Event
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE=ptr
	;
	;	get color indicated by Event.index
	;
	(*PTR).b_color = (*PTR).droplist_colors[Event.index]
	;
	plot_spectrum, Event
end
;
;-----------------------------------------------------------------
pro Toggle_Show_Baseline, Event
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR
	;
	(*PTR).plot_baseline = 1 - (*PTR).plot_baseline
	;
	wid = WIDGET_INFO(Event.top, FIND_BY_UNAME="Show_Baseline_Button")
	WIDGET_CONTROL, wid, SET_BUTTON=(*PTR).plot_baseline
	;
	plot_spectrum, Event
end
;
;-----------------------------------------------------------------
pro Outer_Tab_Change, Event

end
;
;-----------------------------------------------------------------
pro Select_Spectrum_Type, Event

end
;
;----------------------------------------------------------------
pro annotate_lines, Event
	;
	;	based on flags and data values, annotate lines on the spectrum
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR
	;
	;
	;	set up where to plot (top, bottom - based on emission/abs)
	;
	;	TBD****
	;
	;	set up default plot lengths
	;
	;	TBD ***
	;
	;========================================================
	;
	;	check for each type of line, plotting if flag set
	;
	if((*PTR).show_atomic_h eq 1) then show_lines, Event, 'Atomic_Hydrogen'
	if((*PTR).show_atomic_he eq 1) then show_lines, Event, 'Atomic_Helium'
	if((*PTR).show_molecular_h eq 1) then show_lines, Event, 'Molecular_Hydrogen'
	if((*PTR).show_atomic_mg eq 1) then show_lines, Event, 'Atomic_Magnesium'
	if((*PTR).show_atomic_fe eq 1) then show_lines, Event, 'Atomic_Iron'
	if((*PTR).show_atomic_si eq 1) then show_lines, Event, 'Atomic_Silicon'
	if((*PTR).show_atomic_al eq 1) then show_lines, Event, 'Atomic_Aluminum'
	if((*PTR).show_atomic_na eq 1) then show_lines, Event, 'Atomic_Sodium'
	if((*PTR).show_atomic_ca eq 1) then show_lines, Event, 'Atomic_Calcium'
	if((*PTR).show_ionic_fe eq 1) then show_lines, Event, 'Ionic_Iron'
	if((*PTR).show_ionic_n eq 1) then show_lines, Event, 'Ionic_Nitrogen'
	if((*PTR).show_ionic_he eq 1) then show_lines, Event, 'Ionic_Helium'
	if((*PTR).show_molecular_co eq 1) then show_lines, Event, 'Molecular_CO'
	if((*PTR).show_molecular_13co eq 1) then show_lines, Event, 'Molecular_13CO'
	if((*PTR).show_molecular_oh eq 1) then show_lines, Event, 'Molecular_OH'
	;
end
;
;-------------------------------------------------------------------------
pro show_lines, Event, str
	;
	;	plots the lines for a given identifier string, but respect redshift
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR
	;
	redshift = (*PTR).redshift
	;
	left = (*PTR).left / (1.0 + redshift)		; which lines to keep?
	right = (*PTR).right / (1.0 + redshift)
	;
	;	find the correct lines
	;
	n_species = (*PTR).n_species
	;
	ind = WHERE((*(*PTR).species)[0:n_species-1].name eq str,nind)
	if(nind ne 1) then begin
		print, 'Line list for ',str,' not found'
		goto, no_lines
	endif
	;
;	print, 'showing lines for species number ',ind,' ', str
	;
	nlines = (*(*PTR).species)[ind[0]].nlines
	wlines = (*(*PTR).species)[ind[0]].wlines[0:nlines-1]
	clines = (*(*PTR).species)[ind[0]].clines[0:nlines-1]
	;
	color = (*(*PTR).species)[ind[0]].color
	linestyle= (*(*PTR).species)[ind[0]].linestyle
	;
	yrange = (*(*PTR).species)[ind[0]].yrange
	do_label = (*(*PTR).species)[ind[0]].label
	;
	if(do_label eq 0) then clines[*] = ''
	;
	for il = 0, nlines-1 do begin
		;
;		print, ' line ',il,' name = ',clines[il],' rest wave = ',wlines[il]
		;
		if(wlines[il] ge left and wlines[il] le right) then begin
			mark_one_line, Event, wlines[il], COLOR=color, YRANGE=yrange,$
								LINESTYLE=linestyle,LABEL=clines[il]
		endif
	endfor
	;
	cursor_on,Event
	;
	no_lines:
end
;
;-----------------------------------------------------------------
pro mark_one_line, Event, wave, COLOR=color, YRANGE=yrange, LABEL=label,$
				LINESTYLE=linestyle
	;
	;	marks one line on the plot at the indicated wavelength
	;
	;	color is color of line
	;	linestyle
	;	label is the text label
	;	YRANGE is the [start, stop] *fractional* y vector to plot
	;
	;	redshift correction included in line positions
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR
	;
	redshift = (*PTR).redshift
	;
	r_wave = wave * (1.0 + redshift)
	;
;	print, '  redshifted wave = ',r_wave
	;
	;	where on plot to mark in x? (fractional value)
	;
	xwave = x_line(Event,r_wave)
	;
;	print, '  xwave returned = ',xwave
	;
	;	check for yrange
	;
	if(N_ELEMENTS(yrange) eq 0) then begin
		yr = [0.5,0.80]
	endif else begin
		yr = yrange
	endelse
	;
	;	check for color
	;
	if(N_ELEMENTS(color) eq 0) then begin
		this_color = get_color(Event, 'red')
	endif else begin
		this_color = get_color(Event, color)
	endelse
	;
	;	check for linestyle
	;
	if(N_ELEMENTS(linestyle) eq 0) then begin
		ls = 2
	endif else begin
		ls = linestyle
	endelse
	;
	;	check for label
	;
	if(N_ELEMENTS(label) eq 0) then begin
		label_flag = 0
	endif else begin
		label_flag = 1
	endelse
	;
	;	draw line if within 98% of plot width
	;
	pw = (*PTR).position[2] - (*PTR).position[0]
	pw1 = 0.01 * pw
	;
	if(xwave[0] ge (pw1+(*PTR).position[0]) and xwave[1] le ((*PTR).position[2]-pw1)) then begin
		oplot, xwave, yr, psym=-3, symsize=0.001, linestyle=ls, color=this_color, thick=(*PTR).a_thick
		;
		;	write label, if flag set
		;
		if(label_flag eq 1) then begin
			xyouts, xwave[0],yr[1]+0.05,label,color=this_color,alignment=0.0,orientation=(*PTR).a_rot,$
						charthick=(*PTR).a_thick,charsize=(*PTR).a_chsz
		endif
	endif else begin
;		print, 'line outside plot limits'
	endelse
	;
end
;
;-----------------------------------------------------------------
pro Save_Postscript, Event
	;
	;	set flag to write postscript file
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR
	;
	(*PTR).write_postscript_flag = 1
	;
	filter = ['*.ps','*.eps']
	filename = dialog_pickfile(Title='Name of Postscript File',$
		path=(*PTR).dirname, filter=filter)
	;
	;	save the name
	;
	(*PTR).filename_postscript = filename
	;
	;	plot to this file
	;
	plot_spectrum, Event
	;
	result = dialog_message(['Wrote Postscript File',FILE_BASENAME(filename)],/inf)
	;
	;	turn off the plotting flag
	;
	(*PTR).write_postscript_flag = 0
	;
	plot_spectrum, Event
end
;
;-----------------------------------------------------------------
pro Save_JPEG, Event
	;
end
;
;-----------------------------------------------------------------
pro Find_Redshift, Event

end
;---------------------------------------------------------------
pro set_color, Event, name
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR
	;
	ind = WHERE( (*(*PTR).species).name eq name,nind)
	if(nind ne 1) then begin
		print, 'ERROR - could not find species: ',name
	endif else begin
		;
		;	get color indicated by Event.index
		;
		(*(*PTR).species)[ind[0]].color = (*PTR).droplist_colors[Event.index]
		;
		plot_spectrum, Event
	endelse
	;
end
;
;-----------------------------------------------------------------
pro set_label, Event, name
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR
	;
	ind = WHERE( (*(*PTR).species).name eq name,nind)
	if(nind ne 1) then begin
		print, 'ERROR - could not find species: ',name
	endif else begin
		;
		(*(*PTR).species)[ind[0]].label = 1 - (*(*PTR).species)[ind[0]].label
		;
		wid = WIDGET_INFO(Event.top, FIND_BY_UNAME=name+"_Label_Button")
		WIDGET_CONTROL, wid, SET_BUTTON=(*(*PTR).species)[ind[0]].label
		;
		plot_spectrum, Event
	endelse
	;
end
;
;-----------------------------------------------------------------
pro Toggle_Atomic_Hydrogen, Event
;
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR
	;
	(*PTR).show_atomic_h = 1 - (*PTR).show_atomic_h
	;
	wid = WIDGET_INFO(Event.top, FIND_BY_UNAME='Atomic_Hydrogen_Button')
	WIDGET_CONTROL, wid, SET_BUTTON=(*PTR).show_atomic_h
	;
	plot_spectrum, Event
	;
end
;
pro Set_Atomic_Hydrogen_Color, Event
	set_color, Event, 'Atomic_Hydrogen'
end
;
pro Toggle_Atomic_Hydrogen_Labels, Event
	set_label, Event, 'Atomic_Hydrogen'
end
;
;-----------------------------------------------------------------
pro Toggle_Atomic_Helium, Event
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR
	(*PTR).show_atomic_he = 1 - (*PTR).show_atomic_he
	wid = WIDGET_INFO(Event.top, FIND_BY_UNAME='Atomic_Helium_Button')
	WIDGET_CONTROL, wid, SET_BUTTON=(*PTR).show_atomic_he
	plot_spectrum, Event
end
;
pro Set_Atomic_Helium_Color, Event
	set_color, Event, 'Atomic_Helium'
end
;
pro Toggle_Atomic_Helium_Labels, Event
	set_label, Event, 'Atomic_Helium'
end
;
;-----------------------------------------------------------------
pro Toggle_Ionic_Helium, Event
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR
	(*PTR).show_ionic_he = 1 - (*PTR).show_ionic_he
	wid = WIDGET_INFO(Event.top, FIND_BY_UNAME='Ionic_Helium_Button')
	WIDGET_CONTROL, wid, SET_BUTTON=(*PTR).show_ionic_he
	;
	;	helium-II has two buttons, set both
	;
	wid = WIDGET_INFO(Event.top, FIND_BY_UNAME='Ionic_Helium_2_Button')
	WIDGET_CONTROL, wid, SET_BUTTON=(*PTR).show_ionic_he
	plot_spectrum, Event
end
;
pro Set_Ionic_Helium_Color, Event
	set_color, Event, 'Ionic_Helium'
	;
	;	second tab where He-II appears - set droplist for both
	;
	wid = WIDGET_INFO(Event.top, FIND_BY_UNAME="Ionic_Helium_2_Color_Droplist")
	WIDGET_CONTROL, wid, SET_DROPLIST_SELECT=Event.index
	wid = WIDGET_INFO(Event.top, FIND_BY_UNAME="Ionic_Helium_Color_Droplist")
	WIDGET_CONTROL, wid, SET_DROPLIST_SELECT=Event.index
	;
end
;
pro Toggle_Ionic_Helium_Labels, Event
	set_label, Event, 'Ionic_Helium'
	;
	;	second button
	;
	name = 'Ionic_Helium'
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR
	button_name = "Ionic_Helium_2_Label_Button"
	ind = WHERE( (*(*PTR).species).name eq name,nind)
	if(nind ne 1) then begin
		print, 'ERROR - could not find species: ',name
	endif else begin
		;	no extra toggling, though
;		(*(*PTR).species)[ind[0]].label = 1 - (*(*PTR).species)[ind[0]].label
		;
		wid = WIDGET_INFO(Event.top, FIND_BY_UNAME=button_name)
		WIDGET_CONTROL, wid, SET_BUTTON=(*(*PTR).species)[ind[0]].label
		plot_spectrum, Event
	endelse
end
;
;-----------------------------------------------------------------
pro Toggle_Atomic_Silicon, Event
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR
	(*PTR).show_atomic_si = 1 - (*PTR).show_atomic_si
	wid = WIDGET_INFO(Event.top, FIND_BY_UNAME='Atomic_Silicon_Button')
	WIDGET_CONTROL, wid, SET_BUTTON=(*PTR).show_atomic_si
	plot_spectrum, Event
end
;
pro Set_Atomic_Silicon_Color, Event
	set_color, Event, 'Atomic_Silicon'
end
;
pro Toggle_Atomic_Silicon_Labels, Event
	set_label, Event, 'Atomic_Silicon'
end
;
;-----------------------------------------------------------------
pro Toggle_Atomic_Iron, Event
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR
	(*PTR).show_atomic_fe = 1 - (*PTR).show_atomic_fe
	wid = WIDGET_INFO(Event.top, FIND_BY_UNAME='Atomic_Iron_Button')
	WIDGET_CONTROL, wid, SET_BUTTON=(*PTR).show_atomic_fe
	plot_spectrum, Event
end
;
pro Set_Atomic_Iron_Color, Event
	set_color, Event, 'Atomic_Iron'
end
;
pro Toggle_Atomic_Iron_Labels, Event
	set_label, Event, 'Atomic_Iron'
end
;
;-----------------------------------------------------------------
pro Toggle_Atomic_Sodium, Event
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR
	(*PTR).show_atomic_na = 1 - (*PTR).show_atomic_na
	wid = WIDGET_INFO(Event.top, FIND_BY_UNAME='Atomic_Sodium_Button')
	WIDGET_CONTROL, wid, SET_BUTTON=(*PTR).show_atomic_na
	plot_spectrum, Event
end
;
pro Set_Atomic_Sodium_Color, Event
	set_color, Event, 'Atomic_Sodium'
end
;
pro Toggle_Atomic_Sodium_Labels, Event
	set_label, Event, 'Atomic_Sodium'
end
;
;-----------------------------------------------------------------
pro Toggle_Atomic_Calcium, Event
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR
	(*PTR).show_atomic_ca = 1 - (*PTR).show_atomic_ca
	wid = WIDGET_INFO(Event.top, FIND_BY_UNAME='Atomic_Calcium_Button')
	WIDGET_CONTROL, wid, SET_BUTTON=(*PTR).show_atomic_ca
	plot_spectrum, Event
end
;
pro Set_Atomic_Calcium_Color, Event
	set_color, Event, 'Atomic_Calcium'
end
;
pro Toggle_Atomic_Calcium_Labels, Event
	set_label, Event, 'Atomic_Calcium'
end
;
;-----------------------------------------------------------------
pro Toggle_Atomic_Magnesium, Event
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR
	(*PTR).show_atomic_mg = 1 - (*PTR).show_atomic_mg
	wid = WIDGET_INFO(Event.top, FIND_BY_UNAME='Atomic_Magnesium_Button')
	WIDGET_CONTROL, wid, SET_BUTTON=(*PTR).show_atomic_mg
	plot_spectrum, Event
end
;
pro Set_Atomic_Magnesium_Color, Event
	set_color, Event, 'Atomic_Magnesium'
end
;
pro Toggle_Atomic_Magnesium_Labels, Event
	set_label, Event, 'Atomic_Magnesium'
end
;
;-----------------------------------------------------------------
pro Toggle_Atomic_Aluminum, Event
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR
	(*PTR).show_atomic_al = 1 - (*PTR).show_atomic_al
	wid = WIDGET_INFO(Event.top, FIND_BY_UNAME='Atomic_Aluminum_Button')
	WIDGET_CONTROL, wid, SET_BUTTON=(*PTR).show_atomic_al
	plot_spectrum, Event
end
;
pro Set_Atomic_Aluminum_Color, Event
	set_color, Event, 'Atomic_Aluminum'
end
;
pro Toggle_Atomic_Aluminum_Labels, Event
	set_label, Event,'Atomic_Aluminum'
end
;
;-----------------------------------------------------------------
pro Toggle_Molecular_OH, Event
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR
	(*PTR).show_molecular_oh = 1 - (*PTR).show_molecular_oh
	wid = WIDGET_INFO(Event.top, FIND_BY_UNAME='Molecular_OH_Button')
	WIDGET_CONTROL, wid, SET_BUTTON=(*PTR).show_molecular_oh
	plot_spectrum, Event
end
;
pro Set_Molecular_OH_Color, Event
	set_color, Event, 'Molecular_OH'
end
;
pro Toggle_Molecular_OH_Labels, Event
	set_label, Event, 'Molecular_OH'
end
;
;-----------------------------------------------------------------
pro Toggle_Molecular_13CO, Event
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR
	(*PTR).show_molecular_13co = 1 - (*PTR).show_molecular_13co
	wid = WIDGET_INFO(Event.top, FIND_BY_UNAME='Molecular_13CO_Button')
	WIDGET_CONTROL, wid, SET_BUTTON=(*PTR).show_molecular_13co
	plot_spectrum, Event
end
;
pro Set_Molecular_13CO_Color, Event
	set_color, Event, 'Molecular_13CO'
end
;
pro Toggle_Molecular_13CO_Labels, Event
	set_label, Event, 'Molecular_13CO'
end
;
;-----------------------------------------------------------------
pro Toggle_Molecular_CO, Event
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR
	(*PTR).show_molecular_co = 1 - (*PTR).show_molecular_co
	wid = WIDGET_INFO(Event.top, FIND_BY_UNAME='Molecular_CO_Button')
	WIDGET_CONTROL, wid, SET_BUTTON=(*PTR).show_molecular_co
	plot_spectrum, Event
end
;
pro Set_Molecular_CO_Color, Event
	set_color, Event, 'Molecular_CO'
end
;
pro Toggle_Molecular_CO_Labels, Event
    set_label, Event, 'Molecular_CO'
end
;
;-----------------------------------------------------------------
pro Toggle_Molecular_Hydrogen, Event
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR
	(*PTR).show_molecular_h = 1 - (*PTR).show_molecular_h
	wid = WIDGET_INFO(Event.top, FIND_BY_UNAME='Molecular_Hydrogen_Button')
	WIDGET_CONTROL, wid, SET_BUTTON=(*PTR).show_molecular_h
	plot_spectrum, Event
end
;
pro Set_Molecular_Hydrogen_Color, Event
	set_color, Event, 'Molecular_Hydrogen'
end
;
pro Toggle_Molecular_Hydrogen_Labels, Event
	set_label, Event, 'Molecular_Hydrogen'
end
;
;-----------------------------------------------------------------
pro Toggle_Ionic_Nitrogen, Event
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR
	(*PTR).show_ionic_n = 1 - (*PTR).show_ionic_n
	wid = WIDGET_INFO(Event.top, FIND_BY_UNAME='Ionic_Nitrogen_Button')
	WIDGET_CONTROL, wid, SET_BUTTON=(*PTR).show_ionic_n
	plot_spectrum, Event
end
;
pro Set_Ionic_Nitrogen_Color, Event
	set_color, Event, 'Ionic_Nitrogen'
end
;
pro Toggle_Ionic_Nitrogen_Labels, Event
	set_label, Event, 'Ionic_Nitrogen'
end
;
;-----------------------------------------------------------------
pro Toggle_Ionic_Iron, Event
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR
	(*PTR).show_ionic_fe = 1 - (*PTR).show_ionic_fe
	wid = WIDGET_INFO(Event.top, FIND_BY_UNAME="Ionic_Iron_Button")
	WIDGET_CONTROL, wid, SET_BUTTON=(*PTR).show_ionic_fe
	plot_spectrum, Event
end
;
pro Set_Ionic_Iron_Color, Event
	set_color, Event, 'Ionic_Iron'
end
;
pro Toggle_Ionic_Iron_Labels, Event
	set_label, Event, 'Ionic_Iron'
end
;
;-----------------------------------------------------------------
pro Change_Redshift, Event
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR
	;
	;	ignore all until a CR is sensed, then get the full string
	;
	if(Event.ch eq 13B or Event.ch eq 10B) then begin
		;
		WIDGET_CONTROL, Event.id, GET_VALUE=str
		;
		;	check length of string, if zero, then send back a zero
		;
		ch_len = STRLEN(str)
		if(ch_len eq 0) then begin
			(*PTR).redshift = 0.0
			WIDGET_CONTROL, Event.id, SET_VALUE=STRING( (*PTR).redshift, format='(F8.4)')
		endif else begin
			;
			;	else, echo and parse the string
			;
;			print, 'redshift - echo str = ',str
			;
			;	read the string to get the new redshift
			;
			reads, str, redshift, format='(F12.4)'
			;
			(*PTR).redshift = redshift
			WIDGET_CONTROL, Event.id, SET_VALUE=STRING( (*PTR).redshift, format='(3x,F8.4)')
			;
;			print, 'new redshift value = ',redshift
;
			plot_spectrum, Event
			;
		endelse
	endif else begin
;		print, 'not CR, instead = ',Event.ch
	endelse
end
;
;-----------------------------------------------------------------
pro Toggle_Cursor, Event
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR
	;
	(*PTR).cursor_flag = 1 - (*PTR).cursor_flag
	;
	wid = WIDGET_INFO(Event.top, FIND_BY_UNAME='Cursor_Button')
	WIDGET_CONTROL, wid, SET_BUTTON=(*PTR).cursor_flag
	;
	if((*PTR).cursor_flag eq 1) then cursor_on,Event else cursor_off, Event
	;
end
;
;-----------------------------------------------------------------
pro Cool_Stars, Event
	;
	;	toggles cool stars flag. When set, turns on lines
	;	of CO,13CO,SiI,etc
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR
	;
	(*PTR).cool_stars_flag = 1 - (*PTR).cool_stars_flag
	wid = WIDGET_INFO(Event.top, FIND_BY_UNAME="Cool_Stars_Button")
	WIDGET_CONTROL, wid, SET_BUTTON=(*PTR).cool_stars_flag
	;
	if((*PTR).cool_stars_flag eq 1) then begin
		;
		(*PTR).show_molecular_co = 1
		change_color, Event, 'Molecular_CO', 'green'
		;
		(*PTR).show_molecular_13co = 1
		change_color, Event, 'Molecular_13CO', 'cyan'
		;
		(*PTR).show_atomic_si = 1
		change_color, Event, 'Atomic_Silicon','olive'
		;
		(*PTR).show_atomic_fe = 1
		change_color, Event, 'Atomic_Iron','red'
		;
		(*PTR).show_atomic_na = 1
		change_color, Event, 'Atomic_Sodium','green'
		;
		(*PTR).show_atomic_ca = 1
		change_color, Event, 'Atomic_Calcium','blue'
		;
		(*PTR).show_atomic_mg = 1
		change_color, Event, 'Atomic_Magnesium','magenta'
		;
		(*PTR).show_atomic_al = 1
		change_color, Event, 'Atomic_Aluminum','cyan'
		;
	endif else begin
		;
		;	turn off all these lines
		;
		(*PTR).show_molecular_co = 0
		(*PTR).show_molecular_13co = 0
		(*PTR).show_atomic_si = 0
		(*PTR).show_atomic_fe = 0
		(*PTR).show_atomic_na = 0
		(*PTR).show_atomic_ca = 0
		(*PTR).show_atomic_mg = 0
		(*PTR).show_atomic_al = 0
		;
	endelse
	;
	plot_spectrum, Event
	;
end
;
;----------------------------------------------------------------
pro change_color, Event, name, new_color
	;
	;	changes the color of a line species, based on name and color
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR
	;
	ind = WHERE( (*(*PTR).species).name eq name,nind)
	if(nind ne 1) then begin
		print, 'ERROR - could not find species: ',name
	endif else begin
		;
		;	set color indicated by new_color
		;
		(*(*PTR).species)[ind[0]].color = new_color
	endelse
end
;
;-----------------------------------------------------------------
pro Warm_Stars, Event
;
	;	toggles warm stars flag. When set, turns on lines
	;	of H, Si
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR
	;
	(*PTR).warm_stars_flag = 1 - (*PTR).warm_stars_flag
	wid = WIDGET_INFO(Event.top, FIND_BY_UNAME="Warm_Stars_Button")
	WIDGET_CONTROL, wid, SET_BUTTON=(*PTR).warm_stars_flag
	;
	if((*PTR).warm_stars_flag eq 1) then begin
		;
		(*PTR).show_atomic_h = 1
		change_color, Event, 'Atomic_Hydrogen', 'red'
		;
		(*PTR).show_atomic_si = 1
		change_color, Event, 'Atomic_Silicon','olive'
		;
	endif else begin
		;
		;	turn off all these lines
		;
		(*PTR).show_atomic_h = 0
		(*PTR).show_atomic_si = 0
		;
	endelse
	;
	plot_spectrum, Event
	;
end
;
;-----------------------------------------------------------------
pro Hot_Stars, Event
	;
	;	toggles hot stars flag. When set, turns on lines
	;	of H, He
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR
	;
	(*PTR).hot_stars_flag = 1 - (*PTR).hot_stars_flag
	wid = WIDGET_INFO(Event.top, FIND_BY_UNAME="Hot_Stars_Button")
	WIDGET_CONTROL, wid, SET_BUTTON=(*PTR).hot_stars_flag
	;
	if((*PTR).hot_stars_flag eq 1) then begin
		;
		(*PTR).show_atomic_h = 1
		change_color, Event, 'Atomic_Hydrogen', 'red'
		;
		(*PTR).show_atomic_he = 1
		change_color, Event, 'Atomic_Helium', 'blue'
		;
		(*PTR).show_ionic_he = 1
		change_color, Event, 'Ionic_Helium','magenta'
		;
	endif else begin
		;
		;	turn off all these lines
		;
		(*PTR).show_atomic_h = 0
		(*PTR).show_atomic_he = 0
		(*PTR).show_ionic_he = 0
		;
	endelse
	;
	plot_spectrum, Event
	;
end
;
;-----------------------------------------------------------------
pro Emission, Event
	;
	;	toggles emission region flag. When set, turns on lines
	;	of H, He
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR
	;
	(*PTR).emission_flag = 1 - (*PTR).emission_flag
	wid = WIDGET_INFO(Event.top, FIND_BY_UNAME="Emission_Button")
	WIDGET_CONTROL, wid, SET_BUTTON=(*PTR).Emission_flag
	;
	if((*PTR).Emission_flag eq 1) then begin
		;
		(*PTR).show_atomic_h = 1
		change_color, Event, 'Atomic_Hydrogen', 'red'
		;
		(*PTR).show_atomic_he = 1
		change_color, Event, 'Atomic_Helium', 'blue'
		;
		(*PTR).show_ionic_he = 1
		change_color, Event, 'Ionic_Helium','magenta'
		;
		(*PTR).show_ionic_fe = 1
		change_color, Event, 'Ionic_Iron','green'
		;
		(*PTR).show_ionic_n = 1
		change_color, Event, 'Ionic_Nitrogen','olive'
		;
	endif else begin
		;
		;	turn off all these lines
		;
		(*PTR).show_atomic_h = 0
		(*PTR).show_atomic_he = 0
		(*PTR).show_ionic_he = 0
		(*PTR).show_ionic_fe = 0
		(*PTR).show_ionic_n = 0
		;
	endelse
	;
	plot_spectrum, Event
	;
end
;
;-----------------------------------------------------------------
pro Shocks, Event
	;
	;	toggles shock emission region flag. When set, turns on lines
	;	of H2, FeII
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR
	;
	(*PTR).shocks_flag = 1 - (*PTR).shocks_flag
	wid = WIDGET_INFO(Event.top, FIND_BY_UNAME="Shocks_Button")
	WIDGET_CONTROL, wid, SET_BUTTON=(*PTR).shocks_flag
	;
	if((*PTR).shocks_flag eq 1) then begin
		;
		(*PTR).show_molecular_h = 1
		change_color, Event, 'Molecular_Hydrogen', 'magenta'
		;
		(*PTR).show_ionic_fe = 1
		change_color, Event, 'Ionic_Iron', 'blue'
		;
	endif else begin
		;
		;	turn off all these lines
		;
		(*PTR).show_molecular_h = 0
		(*PTR).show_ionic_fe = 0
		;
	endelse
	;
	plot_spectrum, Event
	;
end
